﻿/**
* Class Ball - represents the ball movieclip
**/

class Ball extends MovieClip implements AnimationTickerListener {
	private static var SHOOT_VEL = 4; // Shooting velocity (px/tick)
	private static var MAX_VEL = 7; // maximal possible velocity (px/tick)
	private static var MIN_VEL = 2; // maximal possible velocity (px/tick)
	private static var SPEED_COEFF = .2; //coefficient to adjust the speed of the ball when it hitsa a moving object
	private static var SPEED_DECAY = .15; // Coefficient showing how fast the speed
	                    				// will try to return to "normal"
	private static var SPEED_INCREMENT = .0025; //slowly increase minimal and "normal" speed                    				
	private var velX:Number; // X axis velocity
	private var velY:Number; // Y axis velocity
	private var r:MovieClip; // parent movieclip
	private var normalSpeed:Number;
	
	private var nextX:Number;
	private var nextY:Number;
	
	
	public function getVelX() {
		return velX;
	}
	public function getVelY() {
		return velY;
	}
	
	public function getNextX(){
		return nextX;		
	}

	public function getNextY(){
		return nextY;		
	}
    
	/**
	* Positions the ball at the game field center
	**/

	public  function center(): Void{
		this._x = 160;
		this._y = 120;
	}

	/**
	* Starts the ball movement from the center into the given direction
	* Arguments:
	* degree - direction (in degrees, 0-360)
	**/

	public function shoot(degree:Number): Void{
		r= CastleGame.getRoot();
		var rads = degree * Math.PI/180;
		velX = Ball.SHOOT_VEL*Math.cos(rads);
		velY = Ball.SHOOT_VEL*Math.sin(rads);
		normalSpeed = SHOOT_VEL;
		startMovement();

	}

	/**
	* Starts the ball's animation timer
	**/
	
	public function startMovement():Void{
		AnimationTicker.getInstance().addListener(this);
	}

	/**
	* Stops the animation timer
	**/

	public function stopMovement():Void{
		AnimationTicker.getInstance().removeListener(this);
		
	}
	
	/**
	* Calculates new speed of the ball based on the current speed and the speed of the 
	* object that hit the ball
	* Arguments:
	* oldSpeed - current speed of the ball
	* hitSpeed -  the speed of the object that hit the ball
	* Returns: new speed
	**/

	private function adjustSpeed(oldSpeed:Number, hitSpeed:Number):Number{
		var newSpeedMod = Math.abs(oldSpeed+hitSpeed*SPEED_COEFF);
		if (newSpeedMod>MAX_VEL) { return MAX_VEL;}
		if (newSpeedMod<MIN_VEL) { return MIN_VEL;}
		return oldSpeed+hitSpeed*SPEED_COEFF;
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected upwards
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectUp(hitVel:Number): Void{
		velX=adjustSpeed(velX, hitVel);
		velY = Math.abs(velY);
		velY *= -1;
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected downwards
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectDown(hitVel:Number){
		velX=adjustSpeed(velX, hitVel);
		velY = Math.abs(velY);
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected to the left
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectLeft(hitVel:Number){
		velY=adjustSpeed(velY, hitVel);
		velX = Math.abs(velX);
		velX *= -1;
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected to the right
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectRight(hitVel:Number){
		velY=adjustSpeed(velY, hitVel);
		velX = Math.abs(velX);
	}
	
	/**
	* Animation tick. This function is called by the animation timer repeatedly with the 
	* specified interval. It changes the coordinates of the ball and checks for the collision. 
	**/

	
	public function animTick():Void{
		
		// adjust speed
		var velMod = Math.sqrt(this.velX*this.velX + this.velY*this.velY);
		var sinA = velMod/velY;
		var cosA = velMod/velX;
		//
		if (velMod != normalSpeed) {
			// Adjust speed
			if (Math.abs(velMod-normalSpeed)<SPEED_DECAY) {
				velMod = normalSpeed;
			}
			else {
				// small adjustment
				velMod+=(velMod < normalSpeed)?SPEED_DECAY:-SPEED_DECAY;
			}
		}
		MIN_VEL+=SPEED_INCREMENT;
		MIN_VEL+=SPEED_INCREMENT;
		normalSpeed+=SPEED_INCREMENT;
		
		velX = normalSpeed*cosA;
		velY = normalSpeed*sinA;
		
		this.nextX = this._x + velX;
		this.nextY = this._y + velY;
		// Check for collisions with all players
		if (Player.testCollision(this)) {
			// collision detected
			return;
		}
		
		// Check for collisions with all walls

		if (Wall.testCollision(this)) {
			// collision detected
			return;
		}
		
		// Check for collision with boundary
		if (hitTest(r.top_wall) ){
				this.reflectDown(0);
		}
		if (hitTest(r.left_wall)){
				this.reflectRight(0);
		}
		if (hitTest(r.bottom_wall)){
				this.reflectUp(0);
		}
		if (hitTest(r.right_wall)){
				this.reflectLeft(0);
		}

		this._x = this.nextX;
		this._y = this.nextY;

		updateAfterEvent();
	}
	
}